"""
Zemfrog is a simple framework based on flask for building a REST API quickly.
Which focuses on building a customizable, flexible and manageable REST API!
"""

__author__ = "Aprila Hijriyan"
__email__ = "hijriyan23@gmail.com"
__version__ = "5.0.1"
