<html>

<head>
    <title>Registration</title>
</head>

<body>
    <h1>Registration</h1>
    <div>
        <p>
            Thank you for registering :)
            Confirm your account here:

            <a
                href="http://localhost:3000/account-confirm/{{token}}">http://localhost:3000/account-confirm/{{token}}</a>
        </p>
    </div>
</body>

</html>
